@@ -29,19 +29,18 @@
 #'   the galamm() function.
 #' @srrstats {G2.0a} Secondary documentation of expectations on lengths of
 #'   inputs provided for the parameters in the descriptions below. This applies
-#'   in particular to \code{lambda}, \code{factor}, \code{load.var}, and
-#'   \code{factor_interactions}, as well as \code{family} and
-#'   \code{family_mapping}.
+#'   in particular to \code{lambda}, \code{factor}, \code{load_var}, and
+#'   \code{factor_interactions}, as well as \code{family}.
 #' @srrstats {G2.1} Assertions on types of input implemented in galamm function.
 #' @srrstats {G2.1a} Documentation on expected data types provided for all
 #'   inputs in the documentation below.
 #' @srrstats {G2.2} Assertions on the lengths of arguments are implemented in
 #'   galamm.
 #' @srrstats {G2.3,G2.3a} match.arg() used on "na.action" argument.
-#' @srrstats {G2.3,G2.3b} Arguments "family", "load.var", "factor", and the
+#' @srrstats {G2.3,G2.3b} Arguments "family", "load_var", "factor", and the
 #'   elements of the "start" argument are case sensitive. This is stated in the
 #'   documentation below.
-#' @srrstats {G2.4,G2.4a} Internally, objects family_mapping, weights_mapping
+#' @srrstats {G2.4,G2.4a} Internally, objects weights_mapping
 #'   and lambda_mapping_X are explicitly converted to integer using
 #'   as.integer().
 #' @srrstats {G2.4,G2.4b} as.numeric() used multiple places throughout the code
@@ -142,42 +141,32 @@
 #'   \code{PLmixed} package
 #'   \insertCite{rockwoodEstimatingComplexMeasurement2019}{galamm}.
 #'
-#' @param weights An optional formula object specifying an expression for the
+#' @param dispformula An optional formula object specifying an expression for the
 #'   residual variance. Defaults to \code{NULL}, corresponding to homoscedastic
 #'   errors. The formula is defined in \code{lme4} style; see vignettes and
 #'   examples for details.
 #'
+#' @param weights Deprecated. Use \code{dispformula} instead.
+#'
 #' @param data A data.frame containing all the variables specified by the model
 #'   formula, with the exception of factor loadings.
 #'
-#' @param family A a list or character vector containing one or more model
-#'   families. For each element in \code{family} there should be a corresponding
-#'   element in \code{family_mapping} specifying which elements of the response
-#'   are conditionally distributed according to the given family. Currently
-#'   family can be one of \code{gaussian}, \code{binomial}, and \code{poisson},
-#'   and only canonical link functions are supported. The family arguments can
-#'   either be provided as character values, e.g., \code{c("gaussian",
-#'   "poisson")} or \code{list("gaussian", "poisson")}, as function names, e.g.,
-#'   \code{c(gaussian, poisson)} or \code{list(gaussian, poisson)}, or as
-#'   function calls, e.g., \code{list(gaussian(), poisson())}. In the latter
-#'   case, they must be provided in a list, and bot as a vector. Mixing the
-#'   different ways of describing the family also works, e.g.,
-#'   \code{list("gaussian", poisson())}, but in this case they must be provided
-#'   in a list. When provided as character values, the argument is case
-#'   sensitive.
+#' @param family A family object specifying the response distribution of the
+#'   model. Currently \code{gaussian}, \code{binomial}, and \code{poisson}
+#'   with canonical link functions are supported. Models with mixed response
+#'   types, in which responses are distributed according to one of the three
+#'   supported distributions, can be set up using the [gfam()] function.
 #'
+#' @param family_mapping Deprecated. See the documentation to [gfam()] for how
+#'   to set up mixed response type models.
 #'
-#' @param family_mapping Optional vector mapping from the elements of
-#'   \code{family} to rows of \code{data}. Defaults to \code{rep(1,
-#'   nrow(data))}, which means that all observations are distributed according
-#'   to the first element of \code{family}. The length of \code{family_mapping}
-#'   must be identical to the number of observations, \code{nrow(data)}.
-#'
-#' @param load.var Optional character specifying the name of the variable in
+#' @param load_var Optional character specifying the name of the variable in
 #'   \code{data} identifying what the factors load onto. Default to \code{NULL},
 #'   which means that there are no loading variables. Argument is case
 #'   sensitive.
 #'
+#' @param load.var Deprecated. Use \code{load_var} instead.
+#'
 #' @param lambda Optional factor loading matrix. Numerical values indicate that
 #'   the given value is fixed, while \code{NA} means that the entry is a
 #'   parameter to be estimated. Numerical values can only take the values 0 or
@@ -216,65 +205,7 @@
 #'   \code{\link{galamm_control}}. Defaults to \code{NULL}, which means that the
 #'   defaults of \code{\link{galamm_control}} are used.
 #'
-#' @return A model object of class \code{galamm}, containing the following
-#'   elements:
-#' * \code{call} the matched call used when fitting the model.
-#' * \code{random_effects} a list containing the following two elements:
-#'   * \code{b} random effects in original parametrization.
-#'   * \code{u} random effects standardized to have identity covariance matrix.
-#' * \code{model} a list with various elements related to the model setup and
-#'   fit:
-#'   * \code{deviance} deviance of final model.
-#'   * \code{deviance_residuals} deviance residuals of the final model.
-#'   * \code{df} degrees of freedom of model.
-#'   * \code{family} a list of one or more family objects, as specified in the
-#'   \code{family} arguments to \code{galamm}.
-#'   * \code{factor_interactions} List of formulas specifying interactions
-#'   between latent and observed variables, as provided to the argument
-#'   \code{factor_interactions} to \code{galamm}. If not provided, it is
-#'   \code{NULL}.
-#'   * \code{fit} a numeric vector with fitted values.
-#'   * \code{fit_population} a numeric vector with fitted values excluding
-#'   random effects.
-#'   * \code{hessian} Hessian matrix of final model, i.e., the second
-#'   derivative of the log-likelihood with respect to all model parameters.
-#'   * \code{lmod} Linear model object returned by \code{lme4::lFormula}, which
-#'   is used internally for setting up the models.
-#'   * \code{loglik} Log-likelihood of final model.
-#'   * \code{n} Number of observations.
-#'   * \code{pearson_residual} Pearson residuals of final model.
-#'   * \code{reduced_hessian} Logical specifying whether the full Hessian
-#'   matrix was computed, or a Hessian matrix with derivatives only with respect
-#'   to beta and lambda.
-#'   * \code{response} A numeric vector containing the response values used when
-#'   fitting the model.
-#'   * \code{weights_object} Object with weights used in model fitting. Is
-#'   \code{NULL} when no weights were used.
-#' * \code{parameters} A list object with model parameters and related
-#'   information:
-#'   * \code{beta_inds} Integer vector specifying the indices of fixed
-#'   regression coefficients among the estimated model parameters.
-#'   * \code{dispersion_parameter} One or more dispersion parameters of the
-#'   final model.
-#'   * \code{lambda_dummy} Dummy matrix of factor loadings, which shows the
-#'   structure of the loading matrix that was supplied in the \code{lambda}
-#'   arguments.
-#'   * \code{lambda_inds} Integer vector specifying the indices of factor
-#'   loadings among the estimated model parameters.
-#'   * \code{lambda_interaction_inds} Integer vector specifying the indices
-#'   of regression coefficients for interactions between latent and observed
-#'   variables.
-#'   * \code{parameter_estimates} Numeric vector of final parameter estimates.
-#'   * \code{parameter_names} Names of all parameters estimates.
-#'   * \code{theta_inds} Integer vector specifying the indices of variance
-#'   components among the estimated model parameters. Technically these are the
-#'   entries of the Cholesky decomposition of the covariance matrix.
-#'   * \code{weights_inds} Integer vector specifying the indices of estimated
-#'   weights (used in heteroscedastic Gaussian models) among the estimated model
-#'   parameters.
-#'  * \code{gam} List containing information about smooth terms in the model.
-#'   If no smooth terms are contained in the model, then it is a list of length
-#'   zero.
+#' @return An object of type `galamm` as described in [galammObject].
 #'
 #' @export
 #'
@@ -289,18 +220,15 @@
 #' loading_matrix <- matrix(c(1, NA), ncol = 1)
 #'
 #' # Define mapping to families.
-#' families <- c(gaussian, binomial)
-#' family_mapping <- ifelse(mresp$itemgroup == "a", 1, 2)
-#'
+#' families <- gfam(list(gaussian, binomial))
 #'
 #' # Fit the model
 #' mod <- galamm(
 #'   formula = y ~ x + (0 + level | id),
 #'   data = mresp,
 #'   family = families,
-#'   family_mapping = family_mapping,
 #'   factor = "level",
-#'   load.var = "itemgroup",
+#'   load_var = "itemgroup",
 #'   lambda = loading_matrix
 #' )
 #'
@@ -313,7 +241,7 @@
 #' # We also set the initial value of the random intercept standard deviation
 #' # to 1
 #' mod <- galamm(
-#'   formula = y ~ x + (1 | id), weights = ~ (1 | item),
+#'   formula = y ~ x + (1 | id), dispformula = ~ (1 | item),
 #'   data = hsced, start = list(theta = 1)
 #' )
 #' summary(mod)
@@ -340,7 +268,7 @@
 #'   formula = y ~ 0 + item + sl(x, factor = "loading") +
 #'     (0 + loading | id),
 #'   data = dat,
-#'   load.var = "item",
+#'   load_var = "item",
 #'   lambda = loading_matrix,
 #'   factor = "loading"
 #' )
@@ -360,7 +288,7 @@
 #' mod <- galamm(
 #'   formula = y ~ type + x:response + (0 + loading | id),
 #'   data = latent_covariates,
-#'   load.var = "type",
+#'   load_var = "type",
 #'   lambda = lambda,
 #'   factor = "loading",
 #'   factor_interactions = factor_interactions
@@ -373,12 +301,27 @@
 #' @family modeling functions
 #'
 #' @md
-galamm <- function(formula, weights = NULL, data, family = gaussian,
-                   family_mapping = rep(1, nrow(data)),
-                   load.var = NULL, lambda = NULL, factor = NULL,
-                   factor_interactions = NULL,
+galamm <- function(formula, dispformula = NULL, weights = NULL, data,
+                   family = gaussian, family_mapping = NULL,
+                   load_var = NULL, load.var = NULL, lambda = NULL,
+                   factor = NULL, factor_interactions = NULL,
                    na.action = getOption("na.action"),
                    start = NULL, control = galamm_control()) {
+
+  if (!is.null(load.var)) {
+    warning("`load.var` is deprecated; use `load_var` instead.", call. = FALSE)
+    if (is.null(load_var)) load_var <- load.var
+  }
+  if (!is.null(weights)) {
+    warning("`weights` is deprecated; use `dispformula` instead.",
+            call. = FALSE)
+    if (is.null(dispformula)) dispformula <- weights
+  }
+  if (!is.null(family_mapping)) {
+    warning("`family_mapping` is deprecated; see the gfam() function.",
+            call. = FALSE)
+    # Deal with the case here
+  }
   if (!inherits(data, "data.frame")) {
     stop("data must be a data.frame")
   }
@@ -401,28 +344,25 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   if (!methods::is(formula, "formula")) {
     stop("formula must be a formula")
   }
-  if (!is.null(weights) && !methods::is(weights, "formula")) {
-    stop("weights must be a formula")
-  }
-  if (!is.vector(family_mapping)) {
-    stop("family_mapping must be a vector.")
-  }
-  if (nrow(data) != length(family_mapping)) {
-    stop("family_mapping must contain one index per row in data")
+  if (!is.null(dispformula) && !methods::is(dispformula, "formula")) {
+    stop("dispformula must be a formula")
   }
 
-  family_list <- setup_family(family)
-
-  family_mapping <- as.integer(family_mapping)
-  if (length(family_list) != length(unique(family_mapping))) {
-    stop(
-      "family_mapping must contain a unique index for each element ",
-      "in family_list."
-    )
+  if (inherits(family, "extended.family")) {
+    stop("Unknown family object. Did you use mgcv::gfam()?")
+  }
+  family_list <- if (!inherits(family, "galamm_extended_family")) {
+    if (length(family) == 1 || inherits(family, "family")) {
+      setup_family(family)
+    } else {
+      stop("Use gfam for mixed response types")
+    }
+  } else {
+    family
   }
 
-  if (!is.null(load.var) && (length(load.var) > 1 || !is.character(load.var))) {
-    stop("load.var must be NULL or a character of length one")
+  if (!is.null(load_var) && (length(load_var) > 1 || !is.character(load_var))) {
+    stop("load_var must be NULL or a character of length one")
   }
   if (!is.null(factor) && !is.character(factor)) {
     stop("factor must be NULL or a character vector")
@@ -442,18 +382,18 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
       stop("all non-NA values in lambda must be either 0 or 1")
     }
   }
-  if (any(vapply(list(load.var, lambda, factor), is.null, logical(1))) &&
+  if (any(vapply(list(load_var, lambda, factor), is.null, logical(1))) &&
     any(vapply(
-      list(load.var, lambda, factor),
+      list(load_var, lambda, factor),
       function(x) !is.null(x), logical(1)
     ))) {
     stop(
-      "load.var, lambda, and factor must either all have values or ",
+      "load_var, lambda, and factor must either all have values or ",
       "all be NULL."
     )
   }
 
-  tmp <- setup_factor(load.var, lambda, factor, data)
+  tmp <- setup_factor(load_var, lambda, factor, data)
   data <- tmp$data
   lambda_orig <- tmp$lambda
   rm(tmp)
@@ -465,8 +405,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   gobj <- gamm4(fixed = lme4::nobars(formula), random = rf, data = data)
   colnames(gobj$lmod$X) <- gsub("^X", "", colnames(gobj$lmod$X))
 
-  response_obj <-
-    setup_response_object(family_list, family_mapping, data, gobj)
+  response_obj <- setup_response_object(family_list, data, gobj)
 
   check_matrix <- as.matrix(cbind(response_obj[, 1], gobj$lmod$X))
   rank_check_covs <- qr(check_matrix[, -1, drop = FALSE])$rank
@@ -477,7 +416,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   }
 
   lambda_mappings <- define_factor_mappings(
-    gobj, load.var, lambda_orig, factor, factor_interactions, data
+    gobj, load_var, lambda_orig, factor, factor_interactions, data
   )
 
   lambda <- lambda_mappings$lambda
@@ -487,10 +426,10 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   beta_inds <- max(theta_inds) + seq_along(colnames(gobj$lmod$X))
   lambda_inds <- max(beta_inds) + seq_along(lambda[lambda >= 2])
 
-  if (!is.null(weights)) {
-    weights_obj <- lme4::mkReTrms(lme4::findbars(weights), fr = data)
+  if (!is.null(dispformula)) {
+    weights_obj <- lme4::mkReTrms(lme4::findbars(dispformula), fr = data)
     if (length(weights_obj$flist) > 1) {
-      stop("Multiple grouping terms in weights not yet implemented.")
+      stop("Multiple grouping terms in dispformula not yet implemented.")
     }
     delta <- diff(weights_obj$Zt@p)
     weights_mapping <- as.integer(weights_obj$flist[[1]]) - 2L
@@ -511,6 +450,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
 
   y <- response_obj[, 1]
   trials <- response_obj[, 2]
+  family_mapping <- response_obj[, 3]
   u_init <- rep(0, nrow(gobj$lmod$reTrms$Zt))
   family_txt <- vapply(family_list, function(f) f$family, "a")
   k <- find_k(family_txt, family_mapping, y, trials)
@@ -521,6 +461,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
   )
 
   mlwrapper <- function(par, gradient = FALSE, hessian = FALSE) {
+    tmp <- family_txt
     marginal_likelihood(
       y = y,
       trials = trials,
@@ -538,7 +479,7 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
       weights = par[weights_inds],
       weights_mapping = weights_mapping,
       family = family_txt,
-      family_mapping = family_mapping - 1L,
+      family_mapping = as.integer(family_mapping) - 1L,
       k = k,
       maxit_conditional_modes = maxit_conditional_modes,
       lossvalue_tol = control$pirls_tol_abs,
@@ -615,31 +556,23 @@ galamm <- function(formula, weights = NULL, data, family = gaussian,
     inc_random = FALSE
   )
 
-  pearson_residuals <- (response_obj[, 1] - fit) /
-    unlist(Map(function(x, y) sqrt(family_list[[x]]$variance(y)),
-      x = family_mapping, y = fit
+  pearson_residuals <- (y - fit * trials) /
+    unlist(Map(function(x, y, trials) sqrt(trials * family_list[[x]]$variance(y)),
+      x = family_mapping, y = fit, trials = trials
     ))
 
   if (length(family_list) == 1 && family_list[[1]]$family == "gaussian") {
-    deviance_residuals <- response_obj[, 1] - fit
+    deviance_residuals <- y - fit
     deviance <- -2 * opt$value
   } else {
     # McCullagh and Nelder (1989), page 39
-    tmp <- lapply(
-      family_list,
-      function(x) {
-        x$dev.resids(
-          response_obj[, 1] / response_obj[, 2],
-          fit, response_obj[, 2]
-        )
-      }
-    )
+    tmp <- lapply(family_list, function(x) x$dev.resids(y / trials, fit, trials))
     dev_res <- sqrt(vapply(
       seq_along(family_mapping),
       function(i) tmp[[family_mapping[[i]]]][[i]], 1
     ))
 
-    deviance_residuals <- sign(response_obj[, 1] - fit) * dev_res
+    deviance_residuals <- sign(y - fit * trials) * dev_res
     deviance <- sum((dev_res)^2)
   }
 
