@@ -1,6 +1,6 @@
 #' Add factor variables as data columns
 #'
-#' @param load.var The \code{load.var} argument provided to
+#' @param load_var The \code{load_var} argument provided to
 #'   \code{\link{galamm}}.
 #' @param lambda The \code{lambda} argument provided to \code{\link{galamm}}.
 #' @param factor The \code{factor} argument provided to \code{\link{galamm}}.
@@ -20,16 +20,16 @@
 #' data(KYPSsim, package = "PLmixed")
 #' loading_matrix <- rbind(c(1, 0), c(NA, 0), c(NA, 1), c(NA, NA))
 #' factors <- c("ms", "hs")
-#' load.var <- "time"
-#' res <- setup_factor(load.var, loading_matrix, factors, KYPSsim)
+#' load_var <- "time"
+#' res <- setup_factor(load_var, loading_matrix, factors, KYPSsim)
 #'
 #' # Columns "ms" and "hs" have now been added:
 #' head(res$data)
 #' # NA's in the original formulation have been replaced by 1's:
 #' res$lambda
-setup_factor <- function(load.var, lambda, factor, data) {
-  if (!is.null(load.var) && (!is.character(load.var)) || length(load.var) > 1) {
-    stop("load.var must be a character of length one.")
+setup_factor <- function(load_var, lambda, factor, data) {
+  if (!is.null(load_var) && (!is.character(load_var)) || length(load_var) > 1) {
+    stop("load_var must be a character of length one.")
   }
 
   if (is.null(factor)) {
@@ -40,8 +40,8 @@ setup_factor <- function(load.var, lambda, factor, data) {
   }
 
   eval(parse(text = paste0(
-    "data$", load.var,
-    "<- factor(data$", load.var, ")"
+    "data$", load_var,
+    "<- factor(data$", load_var, ")"
   )))
 
   lambda[is.na(lambda)] <-
@@ -52,15 +52,15 @@ setup_factor <- function(load.var, lambda, factor, data) {
     stop("Factor already a column in data.")
   }
   for (j in seq_along(factor)) {
-    if (length(unique(data[, load.var])) != length(lambda[, j])) {
+    if (length(unique(data[, load_var])) != length(lambda[, j])) {
       stop(
         "lambda matrix must contain one row ",
-        "for each element in load.var"
+        "for each unique element in load_var"
       )
     }
     eval(parse(text = paste("data$", factor[[j]], "<-1")))
     rows_to_zero <-
-      data[, load.var] %in% levels(data[, load.var])[lambda[, j] == 0]
+      data[, load_var] %in% levels(data[, load_var])[lambda[, j] == 0]
     eval(
       parse(
         text =
@@ -100,7 +100,7 @@ setup_factor <- function(load.var, lambda, factor, data) {
 setup_family <- function(family) {
   if (length(family) == 1 || inherits(family, "family")) family <- list(family)
 
-  lapply(family, function(f) {
+  ret <- lapply(family, function(f) {
     if (is.character(f)) {
       return(eval(parse(text = f))())
     } else if (is.function(f)) {
@@ -109,13 +109,19 @@ setup_family <- function(family) {
       return(f)
     }
   })
+
+  lapply(ret, function(f) {
+    if (!inherits(f, "family")) {
+      stop("Not a family object")
+    }
+  })
+
+  ret
 }
 
 #' Create the response object
 #'
 #' @param family_list A list of families returned from \code{setup_family}.
-#' @param family_mapping The argument \code{family_mapping} provided to
-#'   \code{\link{galamm}}.
 #' @param data A data frame.
 #' @param gobj A list object returned from the internal function \code{gamm4}.
 #'
@@ -124,28 +130,43 @@ setup_family <- function(family) {
 #' @return A matrix with responses and number of trials.
 #' @noRd
 #'
-setup_response_object <- function(family_list, family_mapping, data, gobj) {
-  response_obj <- matrix(nrow = nrow(gobj$lmod$X), ncol = 2)
+setup_response_object <- function(family_list, data, gobj) {
+  response_obj <- matrix(nrow = nrow(gobj$lmod$X), ncol = 3)
+  response_name <- all.vars(lme4::nobars(gobj$fake.formula))[[1]]
+
+  if (length(family_list) > 1) {
+    if (!isTRUE(base::all.equal(
+      sort(as.integer(unique(data[, response_name][, 2]))),
+      seq_along(family_list)))) {
+      stop("There must be at least one index per family")
+    }
+  }
 
   for (i in seq_along(family_list)) {
     f <- family_list[[1]]
+    matching_rows <- if (length(family_list) > 1) {
+      data[, response_name][, 2] == i
+    } else {
+      rep(TRUE, nrow(data))
+    }
+
     mf <- stats::model.frame(lme4::nobars(gobj$fake.formula),
-      data = data[family_mapping == i, ]
+      data = data[matching_rows, ]
     )
     mr <- stats::model.response(mf)
 
     if (f$family == "binomial" && !is.null(dim(mr))) {
       trials <- rowSums(mr)
     } else {
-      trials <- rep(1, sum(family_mapping == i))
+      trials <- rep(1, sum(matching_rows))
     }
     if (is.matrix(mr)) {
       response <- mr[, 1, drop = TRUE]
     } else {
       response <- mr
     }
-    response_obj[family_mapping == i, ] <-
-      cbind(response = response, trials = trials)
+    response_obj[matching_rows, ] <-
+      cbind(response = response, trials = trials, family = i)
   }
   response_obj
 }
